module Unicode(Unicode(..), html2unicode,
	       allSymbols) where

import List  (elemIndex)

-- There's no real support for full Unicode here, just for
-- named symbols from HTML 3.2 (and some symbols from HTML 4.0)

allSymbols :: [String]
allSymbols = [htmlSym | (_, xs) <- unicodeSyms, (htmlSym, _) <- xs]

-- LaTeX symbols incomplete; will be filled when the LaTeX backend arrives.
unicodeSyms :: [(Integer, [(String, String)])]
unicodeSyms =
    [(34, [("&quot;", "\""),
	   ("#", undef),
	   ("$", "\\$")
	  ]
     ),
     (60, [("&lt;", "\\ensumremath{<}")]),
     (62, [("&gt;", "\\ensumremath{>}")]),
     (160, [("&nbsp;", "\\ "),
	    ("&iexcl;", undef),
	    ("&cent;", undef),
	    ("&pound;", undef),
	    ("&curren;", undef),
	    ("&yen;", undef),
	    ("&brvbar;", undef),
	    ("&sect;", undef),
	    ("&uml;", undef),
	    ("&copy;", undef),
	    ("&ordf;", undef),
	    ("&laquo;", undef),
	    ("&not;", undef),
	    ("&shy;", undef),
	    ("&reg;", undef),
	    ("&macr;", undef),
	    ("&deg;", undef),
	    ("&plusmn;", undef),
	    ("&sup2;", undef),
	    ("&sup3;", undef),
	    ("&acute;", undef),
	    ("&micro;", undef),
	    ("&para;", undef),
	    ("&middot;", undef),
	    ("&cedil;", undef),
	    ("&sup1;", undef),
	    ("&ordm;", undef),
	    ("&raquo;", undef),
	    ("&frac14;", undef),
	    ("&frac12;", undef),
	    ("&frac34;", undef),
	    ("&iquest;", undef),
	    ("&Agrave;", "\\`A"),        -- 192
	    ("&Aacute;", "\\'A"),
	    ("&Acirc;", undef),
	    ("&Atilde;", undef),
	    ("&Auml;", "\\\"A"),
	    ("&Aring;", undef),
	    ("&AElig;", undef),
	    ("&Ccedil;", undef),
	    ("&Egrave;", "\\`E"),
	    ("&Eacute;", "\\'E"),
	    ("&Ecirc;", undef),
	    ("&Euml;", "\\\"E"),
	    ("&Igrave;", "\\`I"),
	    ("&Iacute;", "\\'I"),
	    ("&Icirc;", undef),
	    ("&Iuml;", "\\\"I"),
	    ("&ETH;", undef),
	    ("&Ntilde;", undef),
	    ("&Ograve;", "\\`O"),
	    ("&Oacute;", "\\'O"),
	    ("&Ocirc;", undef),
	    ("&Otilde;", undef),
	    ("&Ouml;", "\\\"O"),
	    ("&times;", "\\ensuremath{\\times}"),
	    ("&Oslash;", undef),
	    ("&Ugrave;", "\\`U"),
	    ("&Uacute;", "\\'U"),
	    ("&Ucirc;", undef),
	    ("&Uuml;", "\\\"U"),
	    ("&Yacute;", "\\'Y"),
	    ("&THORN;", undef),
	    ("&szlig;", "\\ss"),
	    ("&agrave;", "\\`a"),
	    ("&aacute;", "\\'a"),
	    ("&acirc;", undef),
	    ("&atilde;", undef),
	    ("&auml;", "\\\"A"),
	    ("&aring;", undef),
	    ("&aelig;", undef),
	    ("&ccedil;", undef),
	    ("&egrave;", "\\`e"),
	    ("&eacute;", "\\'e"),
	    ("&ecirc;", undef),
	    ("&euml;", "\\\"e"),
	    ("&igrave;", "\\`i"),
	    ("&iacute;", "\\'A"),
	    ("&icirc;", undef),
	    ("&iuml;", "\\\"i"),
	    ("&eth;", undef),
	    ("&ntilde;", undef),
	    ("&ograve;", "\\`o"),
	    ("&oacute;", "\\'o"),
	    ("&ocirc;", undef),
	    ("&otilde;", undef),
	    ("&ouml;", "\\\"o"),
	    ("&divide;", undef),
	    ("&oslash;", undef),
	    ("&ugrave;", "\\`u"),
	    ("&uacute;", "\\'u"),
	    ("&ucirc;", undef),
	    ("&uuml;", "\\\"u"),
	    ("&yacute;", "\\'y"),
	    ("&thorn;", undef),
	    ("&yuml;", "\\\"y")
	   ]
     ),
     (913, [("&Alpha;", "\\ensuremath{\\Alpha}"),
	    ("&Beta;",  "\\ensuremath{\\Beta}"),
	    ("&Gamma;", "\\ensuremath{\\Gamma}"),
	    ("&Delta;", "\\ensuremath{\\Delta}"),
	    ("&Epsilon", "\\ensuremath{\\Epsilon}"),
	    ("&Zeta;", "\\ensuremath{\\Zeta}"),
	    ("&Eta;", "\\ensuremath{\\Eta}"),
	    ("&Theta;", "\\ensuremath{\\Theta}"),
	    ("&Iota;", "\\ensuremath{\\Iota}"),
	    ("&Kappa;", "\\ensuremath{\\Kappa}"),
	    ("&Lambda;", "\\ensuremath{\\Lambda"),
	    ("&Mu;", "\\ensuremath{\\Mu}"),
	    ("&Nu;", "\\ensuremath{\\Nu}"),
	    ("&Xi;", "\\ensuremath{\\Xi}"),
	    ("&Omicron;", "\\ensuremath{\\Omicron}"),
	    ("&Pi;", "\\ensuremath{\\Pi}"),
	    ("&Rho;", "\\ensuremath{\\Rho}")
	   ]
     ),
-- &Rho; is &#929; and &Sigma; is &#931;, so &#930; is missing here
     (931, [("&Sigma;", "\\ensuremath{\\Sigma}"),
	    ("&Tau;", "\\ensuremath{\\Tau}"),
	    ("&Upsilon;", "\\ensuremath{\\Upsilon}"),
	    ("&Phi;", "\\ensuremath{\\Phi}"),
	    ("&Chi;", "\\ensuremath{\\Chi}"),
	    ("&Psi;", "\\ensuremath{\\Psi}"),
	    ("&Omega;", "\\ensuremath{\\Omega}")]
     ),
     (945, [("&alpha;", "\\ensuremath{\\alpha}"),
	    ("&beta;",  "\\ensuremath{\\beta}"),
	    ("&gamma;", "\\ensuremath{\\gamma}"),
	    ("&delta;", "\\ensuremath{\\delta}"),
	    ("&epsilon", "\\ensuremath{\\epsilon}"),
	    ("&zeta;", "\\ensuremath{\\zeta}"),
	    ("&eta;", "\\ensuremath{\\eta}"),
	    ("&theta;", "\\ensuremath{\\theta}"),
	    ("&iota;", "\\ensuremath{\\iota}"),
	    ("&kappa;", "\\ensuremath{\\kappa}"),
	    ("&lambda;", "\\ensuremath{\\lambda"),
	    ("&mu;", "\\ensuremath{\\mu}"),
	    ("&nu;", "\\ensuremath{\\nu}"),
	    ("&xi;", "\\ensuremath{\\xi}"),
	    ("&omicron;", "\\ensuremath{\\omicron}"),
	    ("&pi;", "\\ensuremath{\\pi}"),
	    ("&rho;", "\\ensuremath{\\rho}"),
	    ("&sigmaf;", undef),                     -- 962
	    ("&sigma;", "\\ensuremath{\\sigma}"),
	    ("&tau;", "\\ensuremath{\\tau}"),
	    ("&upsilon;", "\\ensuremath{\\upsilon}"),
	    ("&phi;", "\\ensuremath{\\varphi}"),
	    ("&chi;", "\\ensuremath{\\chi}"),
	    ("&psi;", "\\ensuremath{\\psi}"),
	    ("&omega;", "\\ensuremath{\\omega}")
	   ]
     ),
     (977, [("&thetasym;", undef),
            ("&upsih;", undef),
            ("&piv", undef)
           ]
     )
    ]


undef = error "some symbol is missing for LaTeX\n (please report this as a bug)"

data Unicode = Unicode { unicodeValue :: Integer, 
			 unicodeHtml :: String,
			 unicodeLatex :: String
		       }
	       deriving (Eq, Show)

html2unicode :: String -> Maybe Unicode
html2unicode str = f unicodeSyms
    where
    f []           = Nothing
    f ((n, xs):us) = case elemIndex str (map fst xs) of
		         Nothing  -> f us
		         Just off -> Just uni
			     where
			     (h, l) = xs !! off
			     uni = Unicode
				   { unicodeValue = n + fromIntegral off,
				     unicodeHtml  = h,
				     unicodeLatex = l }
